<?
$submit_button_text       = "Wygeneruj";
$message_algorithm        = "Algorytm";
$message_amount_length    = "Ilość i długość";
$message_symbol_sets      = "Zestawy znaków";
$message_user_random_seed = "Ziarno użytkownika";
$message_save_settings    = "Zapisz ustawienia w \"cookie\"";
$message_remove_saved     = "Usuń zapisane ustawienia";
$message_command_line     = "Linia polecenia";
$message_generated_pass   = "Wygenerowane hasła";
$message_yes              = "Tak";
$message_no               = "Nie";
$submessage_pronounceable = "Wymawialne:";
$submessage_random        = "Losowe:";
$submessage_num_of_pass   = "Ilość haseł do wygenerowania:";
$submessage_min_pass_len  = "Minimalna długość hasła:";
$submessage_max_pass_len  = "Maksymalna długość hasła:";
$submessage_small_lerrers = "Małe litery:";
$submessage_cap_letters   = "Duże litery:";
$submessage_numbers       = "Liczby";
$submessage_spec_symbols  = "Znaki specjalne:";
$submessage_seed          = "Ziarno:";
$submessage_up_to         = "do 255";
header( "Content-type: text/html; charset=iso-8859-2" );
header ("Pragma: no-cache");
?>
