<?
  ###################################
  # Standard HTML Page colors
  #
  $page_bgcolor = "#000000" ;
  $page_text    = "#ffffff" ;
  $page_link    = "#33ff33" ;
  $page_vlink   = "#33ff33" ;
  $page_alink   = "#ff0000" ;

  ###################################
  # Interface settings
  $interface_width     = "760";
  $copyright_color     = "#333333";
  
  ###################################
  # Pannel properis
  #
  $p_width             = "370";
  $p_border_width      = "1";
  $p_3d_border_width   = "0";
  $p_border_color      = "#cc6600";

  $p_header_bgcolor    = "#990000";
  $p_header_font_color = "#ffffff";
  $p_header_font_size  = "0";
  $p_header_font_face  = "Helvetica,Arial,sans-serif";

  $p_body_bgcolor      = "#000000";
  $p_body_font_color   = "#ffffff";
  $p_body_font_size    = "0";
  $p_body_font_face    = "Helvetica,Arial,sans-serif";
  
  $pass_hilight_color  = "#666666";
?>