<?
$submit_button_text       = "Генерировать";
$message_algorithm        = "Алгоритм";
$message_amount_length    = "Количество и длина";
$message_symbol_sets      = "Наборы символов";
$message_user_random_seed = "Строка иннициализации";
$message_save_settings    = "Сохранить настройки в cookie";
$message_remove_saved     = "Удалить сохраненные настройки";
$message_command_line     = "Командная строка";
$message_generated_pass   = "Сгенерированные пароли";
$message_yes              = "Да";
$message_no               = "Нет";
$submessage_pronounceable = "Произносимые:";
$submessage_random        = "Случайные:";
$submessage_num_of_pass   = "Количество:";
$submessage_min_pass_len  = "Мин. длина пароля:";
$submessage_max_pass_len  = "Макс. длина пароля:";
$submessage_small_lerrers = "Строчные буквы:";
$submessage_cap_letters   = "Заглавные буквы:";
$submessage_numbers       = "Цифры:";
$submessage_spec_symbols  = "Специальные символы:";
$submessage_seed          = "Строка:";
$submessage_up_to         = "до 255";
header( "Content-type: text/html; charset=windows-1251" );
header ("Pragma: no-cache");
?>
